@@ -1,10 +1,10 @@
 
-set rcsId {$Id: demo,v 1.31 1998/04/03 06:12:31 jfontain Exp $}
+set rcsId {$Id: demo,v 1.33 1998/04/20 18:41:37 jfontain Exp $}
 
-if {[catch {package require stooop 3.4}]} {
-set rcsId {$Id: stooop.tcl,v 3.57 1998/03/30 08:35:38 jfontain Exp $}
+if {[catch {package require stooop 3.5.1}]} {
+set rcsId {$Id: stooop.tcl,v 3.60 1998/04/18 20:43:14 jfontain Exp $}
 
-package provide stooop 3.4
+package provide stooop 3.5.1
 
 catch {rename proc _proc}
 
@@ -59,9 +59,8 @@ namespace eval ::stooop {
 
         if {[scan $classOrId %u dummy]==0} {
             set constructor ${classOrId}::[namespace tail $classOrId]
-            set fullName [namespace qualifiers [uplevel namespace which -command $constructor]]
-            set fullClass([set id [incr newId]]) $fullName
-            uplevel $constructor $id $args
+            uplevel $constructor [set id [incr newId]] $args
+            set fullClass($id) [namespace qualifiers [uplevel namespace which -command $constructor]]
         } else {
             if {[catch {set fullClass([set id [incr newId]]) $fullClass($classOrId)}]} {
                 error "invalid object identifier $classOrId"
@@ -396,6 +395,7 @@ if {[llength [array names ::env STOOOP*]]>0} {
 
             if {[scan $classOrId %u dummy]==0} {
                 set constructor ${classOrId}::[namespace tail $classOrId]
+                catch {$constructor}
                 set fullName [namespace qualifiers [uplevel namespace which -command $constructor]]
             } else {
                 set fullName $fullClass($classOrId)
@@ -451,6 +451,7 @@ if {[llength [array names ::env STOOOP*]]>0} {
 
         debugInformation class qualifiedClass procedure qualifiedProcedure this
         if {![info exists this]} return
+        if {[string compare $procedure constructor]==0} return
         if {![info exists fullClass($this)]} {
             error "$this is not a valid object identifier"
         }
@@ -651,17 +652,7 @@ class switched {
 }
 }
 
-if {[catch {package require tkpiechart 4.0}]} {
-set rcsId {$Id: util.tcl,v 1.7 1998/03/14 09:48:05 jfontain Exp $}
-
-set PI 3.14159265358979323846
-
-proc maximum {a b} {
-    return [expr {$a>$b?$a:$b}]
-}
-proc minimum {a b} {
-    return [expr {$a<$b?$a:$b}]
-}
+if {[catch {package require tkpiechart 4.2}]} {
 set rcsId {$Id: pielabel.tcl,v 1.37 1998/03/28 20:42:15 jfontain Exp $}
 
 class pieLabeler {
@@ -687,10 +678,12 @@ class pieLabeler {
     virtual proc selectState {this label {state {}}}
 
 }
-set rcsId {$Id: perilabel.tcl,v 1.35 1998/03/28 20:42:15 jfontain Exp $}
+set rcsId {$Id: perilabel.tcl,v 1.36 1998/04/09 21:55:19 jfontain Exp $}
 
 class piePeripheralLabeler {
 
+    variable PI 3.14159265358979323846
+
     proc piePeripheralLabeler {this canvas args} pieLabeler {$canvas $args} switched {$args} {
         switched::complete $this
     }
@@ -769,7 +762,7 @@ class piePeripheralLabeler {
     }
 
     proc position {this text slice} {
-        global PI
+        variable PI
 
         slice::data $slice data
         set midAngle [expr {$data(start)+($data(extent)/2.0)}]
@@ -1068,10 +1061,12 @@ class canvasLabelsArray {
     }
 
 }
-set rcsId {$Id: slice.tcl,v 1.31 1998/03/25 20:45:16 jfontain Exp $}
+set rcsId {$Id: slice.tcl,v 1.32 1998/04/09 21:56:03 jfontain Exp $}
 
 
-class slice {}
+class slice {
+    variable PI 3.14159265358979323846
+}
 
 proc slice::slice {this canvas x y radiusX radiusY start extent args} switched {$args} {
 
@@ -1180,7 +1175,7 @@ proc slice::update {this start extent} {
 }
 
 proc slice::updateBottom {this} {
-    global PI
+    variable PI
 
     set start $slice::($this,start)
     set extent $slice::($this,extent)
@@ -1293,9 +1288,152 @@ proc slice::data {this arrayName} {
     set data(yCenter) [expr {[lindex $coordinates 1]+$data(yRadius)}]
     set data(height) $switched::($this,-height)
 }
-set rcsId {$Id: pie.tcl,v 1.60 1998/03/28 20:42:39 jfontain Exp $}
 
-package provide tkpiechart 4.0
+class slice {
+    proc maximum {a b} {return [expr {$a>$b?$a:$b}]}
+}
+
+set rcsId {$Id: objselec.tcl,v 1.3 1998/04/20 14:51:14 jfontain Exp $}
+
+
+class objectSelection {
+
+    proc objectSelection {this args} switched {$args} {
+        switched::complete $this
+    }
+
+    proc ~objectSelection {this} {
+        variable ${this}selected
+
+        catch {::unset ${this}selected}
+    }
+
+    proc options {this} {
+        return [::list\
+            [::list -selectcommand {} {}]\
+        ]
+    }
+
+    proc set-selectcommand {this value} {}
+
+    proc set {this ids selected} {
+        variable ${this}selected
+
+        ::set select {}
+        ::set deselect {}
+        foreach id $ids {
+            if {[info exists ${this}selected($id)]&&($selected==[::set ${this}selected($id)])} continue
+            if {$selected} {
+                lappend select $id
+                ::set ${this}selected($id) 1
+            } else {
+                lappend deselect $id
+                ::set ${this}selected($id) 0
+            }
+        }
+        update $this $select $deselect
+    }
+
+    proc update {this selected deselected} {
+        if {[string length $switched::($this,-selectcommand)]==0} return
+        if {[llength $selected]>0} {
+            uplevel #0 $switched::($this,-selectcommand) [::list $selected] 1
+        }
+        if {[llength $deselected]>0} {
+            uplevel #0 $switched::($this,-selectcommand) [::list $deselected] 0
+        }
+    }
+
+    proc unset {this ids} {
+        variable ${this}selected
+
+        foreach id $ids {
+            ::unset ${this}selected($id)
+        }
+    }
+
+
+    proc add {this ids} {
+        set $this $ids 0
+    }
+
+    proc remove {this ids} {
+        unset $this $ids
+    }
+
+    proc select {this ids} {
+        clear $this
+        set $this $ids 1
+        ::set objectSelection::($this,lastSelected) [lindex $ids end]
+    }
+
+    proc deselect {this ids} {
+        set $this $ids 0
+    }
+
+    proc toggle {this ids} {
+        variable ${this}selected
+
+        ::set select {}
+        ::set deselect {}
+        foreach id $ids {
+            if {[::set ${this}selected($id)]} {
+                lappend deselect $id
+                ::set ${this}selected($id) 0
+            } else {
+                lappend select $id
+                ::set ${this}selected($id) 1
+                ::set objectSelection::($this,lastSelected) $id
+            }
+        }
+        update $this $select $deselect
+    }
+
+    proc extend {this id} {
+        variable ${this}selected
+
+        if {[info exists objectSelection::($this,lastSelected)]} {
+            ::set list [lsort -integer [array names ${this}selected]]
+            ::set last [lsearch -exact $list $objectSelection::($this,lastSelected)]
+            ::set index [lsearch -exact $list $id]
+            clear $this
+            if {$index>$last} {
+                set $this [lrange $list $last $index] 1
+            } else {
+                set $this [lrange $list $index $last] 1
+            }
+        } else {
+            select $this $id
+        }
+    }
+
+    proc clear {this} {
+        variable ${this}selected
+
+        set $this [array names ${this}selected] 0
+    }
+
+    proc selected {this} {
+        variable ${this}selected
+
+        ::set list {}
+        foreach id [array names ${this}selected] {
+            if {[::set ${this}selected($id)]} {
+                lappend list $id
+            }
+        }
+        return [lsort -integer $list]
+    }
+
+    proc list {this} {
+        variable ${this}selected
+
+        return [lsort -integer [array names ${this}selected]]
+    }
+}
+set rcsId {$Id: pie.tcl,v 1.64 1998/04/20 20:17:25 jfontain Exp $}
+
+package provide tkpiechart 4.2
 
 class pie {
     set pie::(colors) {#7FFFFF #7FFF7F #FF7F7F #FFFF7F #7F7FFF #FFBF00 #BFBFBF #FF7FFF #FFFFFF}
@@ -1320,6 +1458,7 @@ proc pie::~pie {this} {
     catch {$pie::($this,canvas) delete $pie::($this,title)}
     delete $pie::($this,labeler)
     eval delete $pie::($this,slices) $pie::($this,backgroundSlice)
+    catch {delete $pie::($this,selector)}
 }
 
 proc pie::options {this} {
@@ -1404,9 +1543,17 @@ proc pie::newSlice {this {text {}}} {
     $canvas addtag pie($this) withtag pieLabeler($labeler)
 
     if {$switched::($this,-selectable)} {
-        $canvas bind canvasLabel($label) <ButtonRelease-1>\
-            "pieLabeler::selectState $labeler $label \[expr {!\[pieLabeler::selectState $labeler $label\]}\]"
-        $canvas bind slice($slice) <ButtonRelease-1> [$canvas bind canvasLabel($label) <ButtonRelease-1>]
+        if {![info exists pie::($this,selector)]} {
+            set pie::($this,selector) [new objectSelection -selectcommand "pie::setLabelsState $this"]
+        }
+        set selector $pie::($this,selector)
+        objectSelection::add $selector $label
+        $canvas bind canvasLabel($label) <ButtonRelease-1> "objectSelection::select $selector $label"
+        $canvas bind slice($slice) <ButtonRelease-1> "objectSelection::select $selector $label"
+        $canvas bind canvasLabel($label) <Control-ButtonRelease-1> "objectSelection::toggle $selector $label"
+        $canvas bind slice($slice) <Control-ButtonRelease-1> "objectSelection::toggle $selector $label"
+        $canvas bind canvasLabel($label) <Shift-ButtonRelease-1> "objectSelection::extend $selector $label"
+        $canvas bind slice($slice) <Shift-ButtonRelease-1> "objectSelection::extend $selector $label"
     }
 
     return $slice
@@ -1424,6 +1571,9 @@ proc pie::deleteSlice {this slice} {
         slice::rotate $following $extent
     }
     pieLabeler::delete $pie::($this,labeler) $pie::($this,sliceLabel,$slice)
+    if {$switched::($this,-selectable)} {
+        objectSelection::remove $pie::($this,selector) $pie::($this,sliceLabel,$slice)
+    }
     unset pie::($this,sliceLabel,$slice)
 }
 
@@ -1472,6 +1622,18 @@ proc pie::selectedSlices {this} {
     }
     return $list
 }
+
+proc pie::setLabelsState {this labels selected} {
+    set labeler $pie::($this,labeler)
+    foreach label $labels {
+        pieLabeler::selectState $labeler $label $selected
+    }
+}
+
+class pie {
+    proc maximum {a b} {return [expr {$a>$b?$a:$b}]}
+    proc minimum {a b} {return [expr {$a<$b?$a:$b}]}
+}
 }
 
 pack [label .m -relief sunken -text\
@@ -1484,7 +1646,7 @@ pack $canvas -fill both -expand 1
 set pie1 [new pie\
     $canvas 0 0 200 100 -thickness 20 -background gray\
     -labeler [new pieBoxLabeler $canvas -justify center -offset 10]\
-    -title "this is pie #1" -titlefont fixed\
+    -title "this is pie #1" -titlefont fixed -selectable 1\
 ]
 set slice11 [pie::newSlice $pie1]
 set slice12 [pie::newSlice $pie1]
