@@ -2,7 +2,7 @@
 # the next line restarts using the interpreter \
 exec tclsh "$0" "$@"
 
-# $Id: instapkg.tcl,v 1.4 1998/04/03 06:13:22 jfontain Exp $
+# $Id: instapkg.tcl,v 1.5 1998/04/13 22:19:16 jfontain Exp $
 
 if {[llength $argv]!=1} {
     puts stderr "usage: $argv0 directory"
@@ -22,16 +22,12 @@ if {![file exists $directory]} {
     exit 1
 }
 
-if {[catch {package require stooop 3.4} message]} {
+if {[catch {package require stooop 3.5} message]} {
     puts stderr "stooop package must be installed first: $message"
     exit 1
 }
-if {[catch {package require switched 1.4} message]} {
-    puts stderr "switched package must be installed first: $message"
-    exit 1
-}
 
-if {[catch {eval file copy -force {pkgIndex.tcl util.tcl pielabel.tcl perilabel.tcl boxlabel.tcl canlabel.tcl labarray.tcl slice.tcl pie.tcl} {$directory}} message]} {
+if {[catch {eval file copy -force {pkgIndex.tcl pie.tcl slice.tcl pielabel.tcl boxlabel.tcl canlabel.tcl perilabel.tcl labarray.tcl objselec.tcl} {$directory}} message]} {
     puts stderr $message
     exit 1
 }
